@@ -13,20 +13,29 @@ gts <- function(y, groups, gnames = rownames(groups), characters) {
   #
   # Error handling:
   if (!is.ts(y)) {
-    y <- as.ts(y)
+    y <- stats::as.ts(y)
   }
 
   if (ncol(y) <= 1L) {
     stop("Argument y must be a multivariate time series.")
   }
   bnames <- colnames(y)
+  nc.y <- ncol(y)
   if (missing(characters)) {
     if (missing(groups)) {
-      groups <- matrix(c(rep(1L, ncol(y)), seq(1L, ncol(y))), nrow = 2L, 
+      groups <- matrix(c(rep(1L, nc.y), seq(1L, nc.y)), nrow = 2L,
                      byrow = TRUE)
     } else if (!is.matrix(groups)) {
       stop("Argument groups must be a matrix.")
-    } 
+    } else if (!is.character(groups[1L, ])) { # Check groups numeric matrix
+      if (all(groups[1L, ] == 1L)) { # if the first row is all 1's
+        groups <- groups[-1L, , drop = FALSE]
+      }
+      tmp.last <- nrow(groups)
+      if (all(groups[tmp.last, ] == seq(1L, nc.y))) { # if the last row is a seq
+        groups <- groups[-tmp.last, , drop = FALSE]
+      }
+    }
     # Check whether groups is unique
     # But R takes so long to check due to the inefficiency with strings
     # bgroup <- unique(apply(groups, 2, paste, collapse = ""))
@@ -55,7 +64,7 @@ gts <- function(y, groups, gnames = rownames(groups), characters) {
   } else if (is.null(gnames)) {
     message("Argument gnames is missing and the default labels are used.")
     gnames <- paste0("G", 1L:(nr.gmat - 2L))
-  } 
+  }
   colnames(gmat) <- bnames
   rownames(gmat) <- c("Total", gnames, "Bottom")
 
@@ -67,7 +76,7 @@ gts <- function(y, groups, gnames = rownames(groups), characters) {
     if (is.matrix(subnames)) {
       # Convert a matrix to a list
       subnames <- split(subnames, rep(1L:ncol(subnames), each = nrow(subnames)))
-    } 
+    }
     name.list <- mapply(paste0, full.groups, "/", subnames, SIMPLIFY = FALSE)
     names(name.list) <- gnames
   }
@@ -199,12 +208,12 @@ print.gts <- function(x, ...) {
     cat("Number of observations per series:", nrow(x$bts), "\n")
     cat("Top level series: \n")
   } else {
-    cat("Number of observations in each historical series:", 
+    cat("Number of observations in each historical series:",
         nrow(x$histy), "\n")
     cat("Number of forecasts per series:", nrow(x$bts), "\n")
     cat("Top level series of forecasts: \n")
   }
-  topts <- ts(rowSums(x$bts, na.rm = TRUE), start = tsp(x$bts)[1L], 
-              frequency = tsp(x$bts)[3L])
+  topts <- ts(rowSums(x$bts, na.rm = TRUE), start = stats::tsp(x$bts)[1L],
+              frequency = stats::tsp(x$bts)[3L])
   print(topts)
 }
