@@ -26,8 +26,8 @@ plot.gts <- function(x, include, levels, labels = TRUE, ...) {
     histx <- histx
     include <- nrow(histx)
   } else {
-    tspx <- tsp(histx)
-    histx <- window(histx, start = tspx[2L] - include/tspx[3L] + 1L/tspx[3L])
+    tspx <- stats::tsp(histx)
+    histx <- stats::window(histx, start = tspx[2L] - include/tspx[3L] + 1L/tspx[3L])
   }
 
   if (missing(levels)) {
@@ -56,17 +56,17 @@ plot.gts <- function(x, include, levels, labels = TRUE, ...) {
   }
 
   cs <- c(0L, cumsum(m))
-  
-  for (i in 1L:l.levels) { 
+
+  for (i in 1L:l.levels) {
     end <- cs[i + 1L]
     start <- cs[i] + 1L
     series <- seq(start, end)
-    cols <- rainbow(length(series))
+    cols <- grDevices::rainbow(length(series))
     if(!is.null(x$histy)) {
       ylim <- range(histx[, series], fcasts[, series], na.rm = TRUE)
       if (labels) {
         strlabels <- max(strwidth(x$labels[levels], units = "figure"))
-        xlim <- range(time(histx)[1L] - strlabels, time(fcasts), 
+        xlim <- range(time(histx)[1L] - strlabels, time(fcasts),
                       na.rm = TRUE)
       } else {
         xlim <- range(time(histx), time(fcasts), na.rm = TRUE)
@@ -82,29 +82,29 @@ plot.gts <- function(x, include, levels, labels = TRUE, ...) {
       }
     }
     if (is.null(dots.list$xlim)) {
-      plot(histx[, series, drop = FALSE], col = cols, xlim = xlim, ylim = ylim, 
-           xlab = "", ylab = "", main = names(x$labels)[levels][i], 
-           plot.type = "single", 
-           type = ifelse(length(1:include) == 1L, "p", "l"), 
+      plot(histx[, series, drop = FALSE], col = cols, xlim = xlim, ylim = ylim,
+           xlab = "", ylab = "", main = names(x$labels)[levels][i],
+           plot.type = "single",
+           type = ifelse(length(1:include) == 1L, "p", "l"),
            ...)
     } else {
-      plot(histx[, series, drop = FALSE], col = cols, ylim = ylim, 
-           xlab = "", ylab = "", main = names(x$labels)[levels][i], 
-           plot.type = "single", 
-           type = ifelse(length(1:include) == 1L, "p", "l"), 
+      plot(histx[, series, drop = FALSE], col = cols, ylim = ylim,
+           xlab = "", ylab = "", main = names(x$labels)[levels][i],
+           plot.type = "single",
+           type = ifelse(length(1:include) == 1L, "p", "l"),
            ...)
     }
 
     if (!is.null(x$histy)) {
       for (j in 1L:length(series)) {
-        lines(fcasts[, series[j], drop = FALSE], lty = 2, col = cols[j], 
+        lines(fcasts[, series[j], drop = FALSE], lty = 2, col = cols[j],
               type = ifelse(nrow(fcasts) == 1L, "p", "l"))
       }
     }
 
     if (labels) {
-      text(x = tsp(histx)[1L] + 0.1, y = histx[1L, series] + 0.2,
-           labels = unlist(x$labels[levels][i]), 
+      text(x = stats::tsp(histx)[1L] + 0.1, y = histx[1L, series] + 0.2,
+           labels = unlist(x$labels[levels][i]),
            cex = 0.9, adj = 1)
     }
   }
