@@ -1,6 +1,6 @@
 window.gts <- function(x, ...) {
   # Select a snapshot of hts or gts
-  x$bts <- window(x$bts, ...)
-  tsp(x) <- tsp(x)
+  x$bts <- stats::window(x$bts, ...)
+  tsp(x) <- stats::tsp(x)
   return(x)
 }
